#include "ROctave.h"

#include "ov-struct.h"

USER_OBJECT_
toRValue(octave_value val)
{
   USER_OBJECT_ sval = NULL_USER_OBJECT;
   int i, n;

//   cerr << "Type name: " << val.type_name() << endl;

   if(strcmp(val.type_name().c_str(), "RReference") == 0) {
     /*     val.rep->ref(); */
     sval =  ((ROctaveObjectReference*)&val)->ref();
   } else if(val.is_string()) {
     PROTECT(sval = NEW_CHARACTER(1));
     SET_STRING_ELT(sval, 0, COPY_TO_USER_STRING(val.string_value().c_str()));
     UNPROTECT(1);
   } else if(val.is_real_scalar()) {
      sval = NEW_NUMERIC(1);
      REAL(sval)[0] = val.double_value();
   } else if(val.is_real_matrix()) {
      Matrix m;
      MArray<double> a;
      m = val.matrix_value();
      if(m.rows()  == 1) {
        a = m.row(0);
      } else if(m.columns() == 1) {
        a = m.column(0);
      } else {
         int nr, nc, ctr, j;
	 nr = m.rows();
	 nc = m.columns();
	 sval = allocMatrix(REALSXP, nr, nc);
	 ctr = 0;
	 for(j = 0; j < nc; j++) {
   	    a = m.column(j);
   	    for(i = 0; i < nc ; i++) {
	       	 REAL(sval)[ctr++] = a(i);
	    }
	 }
         return(sval);
      }

      n = a.length();
      sval = NEW_NUMERIC(n);
      for(i = 0 ; i < n; i++) {
          REAL(sval)[i] = a(i);
      }
   }
 
   return(sval);
}


octave_value
toOctaveValue(USER_OBJECT_ robj)
{
    octave_value val;
    int i, n;
    USER_OBJECT_ dims, names;
    dims = GET_DIM(robj);
    names = GET_NAMES(robj);

    /* If it has a dimension */
    if(GET_LENGTH(dims)) {
      int nr, nc, j, ctr = 0;
      nr = INTEGER(dims)[0];
      nc = INTEGER(dims)[1];
      Matrix m = Matrix(nr, nc);
      for(j = 0; j < nc; j++) {
        for(i = 0; i < nr; i++) {
          if(TYPEOF(robj) == REALSXP)
            m(i, j) = REAL(robj)[ctr];
          else if(TYPEOF(robj) == INTSXP)
            m(i, j) = INTEGER(robj)[ctr];
          else if(TYPEOF(robj) == LGLSXP)
            m(i, j) = LOGICAL(robj)[ctr];
          ctr++;
        }
      }
      val = m;
      return(val);
    }

        /* Create an Octave struct by mapping each of the named 
           elements in the R object to the same name in the Octave
           structure and working recursively. 
           Should also take care of attributes; perhaps a sub-list of 
           elements.
         */
    if(TYPEOF(robj) == VECSXP) {
     if(GET_LENGTH(names)) {
      n = GET_LENGTH(names);
      Octave_map m;
      for(i = 0; i < n; i++) {
        m[CHAR_DEREF(STRING_ELT(names, i))] = toOctaveValue(VECTOR_ELT(robj, i));
      }
      val = octave_value(m);
      return(val);
     } else {
#if 0
      n = GET_LENGTH(robj);
      octave_value_list v;
      for(i = 0; i < n; i++) {
  	v(i) = toOctaveValue(VECTOR_ELT(robj, i));
      }
      val = octave_value(v);
     return(val);
#endif
     }
    }

    if(TYPEOF(robj) == REALSXP) {
	   n = Rf_length(robj);
           RowVector v = RowVector(n);
	   for(i = 0; i < n; i++) {
  	       v(i) = REAL(robj)[i];
	   }
	   val = v;
    } else if(TYPEOF(robj) == INTSXP) {
	   n = Rf_length(robj);
           RowVector v = RowVector(n);
	   for(i = 0; i < n; i++) {
  	       v(i) = INTEGER(robj)[i];
	   }
	   val = v;
    } else if(TYPEOF(robj) == LGLSXP) {
	   n = Rf_length(robj);
           RowVector v = RowVector(n);
	   for(i = 0; i < n; i++) {
  	       v(i) = LOGICAL(robj)[i];
	   }
	   val = v;
    } else if(TYPEOF(robj) == STRSXP) {
	   n = Rf_length(robj);
           if(n == 1)
             val = string(CHAR_DEREF(STRING_ELT(robj, 0)));
           else {
             string_vector v(n);
             for(i = 0; i < n; i++) {
  	       v(i) = string(CHAR_DEREF(STRING_ELT(robj, i)));
             }
             val = v;
           }
    } else {
        /* Create a reference to the R object. 
           If we are dealing with a closure, treat as an ROctaveFunctionReference.
           Otherwise treat as a generic R reference object. */
        /* Not quite working yet. */
      if(TYPEOF(robj) == CLOSXP)
         val = new ROctaveFunctionReference(robj);
      else
         val = new ROctaveObjectReference(robj);
    }

    return(val);
}
