#include "RSCommon.h"

#include <octave/oct.h>
#include <octave/ov-base.h>
#include <octave/ov-scalar.h>

extern "C" {
  /* Temporarily get human readable names for easy debugging. There is no other need
     to have these protected from name mangling. */
USER_OBJECT_ ROctave_getFun(octave_value funName);
USER_OBJECT_ toRValue(octave_value val);
octave_value toOctaveValue(USER_OBJECT_ robj);
}



extern "C" {
   void R_PreserveObject(USER_OBJECT_ val);
}

/* A class of Octave object that holds a reference to an R object. */
class ROctaveObjectReference : public octave_base_value
{
   protected:
     USER_OBJECT_ _ref;

   public:

    ~ROctaveObjectReference(void) { }
    ROctaveObjectReference(octave_value &a) {
       _ref = ((ROctaveObjectReference *)&a)->_ref;
    }
    ROctaveObjectReference(ROctaveObjectReference &a) {
       _ref = a._ref;
    }

    octave_value *clone (void) { return new ROctaveObjectReference(*this); }
   
     ROctaveObjectReference(USER_OBJECT_ val) : octave_base_value() {
        R_PreserveObject(val);
        ref(val);
     }

//   void *operator new (size_t size){ return allocator.alloc (size); }
//   void operator delete (void *p, size_t size){ allocator.free (p, size); }

   void print (ostream& os, bool pr_as_read_syntax = false) {
      os << "RReference" << " <" << (void *) _ref << ">";
   }

   bool is_defined (void) const { return true; }
   bool is_real_type (void) const { return true; }

   USER_OBJECT_ ref(USER_OBJECT_ val = NULL) {
	 if(val) {
           _ref = val;
	 }

         return(_ref);
     }

 protected:

  // For custom memory management.
//  static octave_allocator allocator;

  static void register_type (void)
    { t_id = octave_value_typeinfo::register_type (t_name); }

  string type_name (void) const { return "RReference"; }

  static int t_id;
  static const string t_name;
};


class ROctaveFunctionReference : public ROctaveObjectReference
{
 public:
   ROctaveFunctionReference(USER_OBJECT_ val) : ROctaveObjectReference(val) {
   }

   void print (ostream& os, bool pr_as_read_syntax = false) {
      os << "RFunctionReference" << " <" << (void *) _ref << ">";
   }
 
  protected:  
  static void register_type (void)
    { t_id = octave_value_typeinfo::register_type (t_name); }

  string type_name (void) const { return "RFunctionReference"; }

  static int t_id;
  static const string t_name;
};

