#include "ROctave.h"

// const string ROctaveObjectReference::t_name ("RReference");


static Rboolean R_initialized = (Rboolean) FALSE;

extern "C" {
 int Rf_initEmbeddedR(int argc, const char * const *argv);
}

octave_value
initialize(const char * const *argv = NULL, int argc = 0)
{
    octave_value val;
    const char * const DefaultArgs[] = {"ROctave"};
    if(!argv) {
       argv = DefaultArgs;
       argc = sizeof(DefaultArgs)/sizeof(DefaultArgs[0]);
    }
    Rf_initEmbeddedR(argc, argv);

    R_initialized = (Rboolean) TRUE;

    return(val);
}

/*
 This is the primary mechanism for calling an S function.
 The argument list should be function name, argument, argument, ...
*/
DEFUN_DLD (ROctave_callR, args, , "call an R function from Octave")
{
  octave_value ans;
  USER_OBJECT_ e, tmp, val;
  int nargs, i;
  int errorOccurred = 0;

  if(!R_initialized) {
     initialize();
  }

  nargs = args.length();
  PROTECT(e = allocVector(LANGSXP, nargs));
  SETCAR(e, ROctave_getFun(args(0)));

  tmp = CDR(e);
  for(i = 1; i < nargs ; i++) {
     SETCAR(tmp, toRValue(args(i)));
     tmp = CDR(tmp);
  }


  PROTECT(val = R_tryEval(e, R_GlobalEnv, &errorOccurred));
  if(!errorOccurred) {
     ans = toOctaveValue(val);
  } else {
	  /* Raise exception, error, etc. */
    //    ::error("%s", "An error occurred when calling an R function");
  }
  return(ans);
}


DEFUN_DLD (ROctave_namedCallR, args, , "call an R function from Octave with named arguments")
{
  octave_value ans;
  USER_OBJECT_ e, tmp, val;
  int nargs, i;
  int errorOccurred = 0;

  if(!R_initialized) {
     initialize();
  }

  nargs = args.length();
  if((nargs-1)%2 != 0) {
    ::error("number of arguments must be even");    
  }
  PROTECT(e = allocVector(LANGSXP, (nargs-1)/2 + 1));
  SETCAR(e, ROctave_getFun(args(0)));

  tmp = CDR(e);
  for(i = 1; i < nargs ; i+=2) {
     const char *tempName;
     string str;
     SETCAR(tmp, toRValue(args(i+1)));
     str = args(i).string_value();
     if(str.length() > 0 && (tempName = str.c_str()))
       SET_TAG(tmp, Rf_install(tempName));
     tmp = CDR(tmp);
  }


  PROTECT(val = R_tryEval(e, R_GlobalEnv, &errorOccurred));
  if(!errorOccurred) {
     ans = toOctaveValue(val);
  } else {
	  /* Raise exception, error, etc. */
    //    ::error("%s", "An error occurred when calling an R function");
  }
  return(ans);
}

USER_OBJECT_
ROctave_getFun(octave_value funName)
{
  char* foo;
  if(funName.is_undefined())
    ::error("%s", "Invalid function name/object passed to ROctave_getFun()");

  foo = (char *) funName.string_value().c_str();
  return(Rf_install(foo));
}


